/************************************************************************
 * Chutes and Ladders in JavaScript										*
 * An online version of the classic family board game. Add multiple 	*
 * players or one or more CPU players, or just watch the AI play 		*
 * against itself. The user can also increase the AI's aggressive		*	
 * playability by toggling a more competitive AI setting, increasing 	*
 * the gameplay's rate of speed, and turning background music on or		*
 * off all in real-time with sleek animation effects for movement.		*
 * Written by Chad Jordan												*
 * 11/12/2010															*
 ***********************************************************************/

var pic, rollr, diffX, diffY;
var players = new Array();
var colors = new Array("blue", "green", "orange", "purple", "red", "yellow", "aqua", "black", "pink");
var chutes = new Array(16, 10, 47, 21, 49, 38, 56, 3, 62,43, 64, 4, 87, 63, 93, 20, 95, 20, 98, 20);
var ladders = new Array(1, 37, 4, 10, 9, 22, 28, 56, 21, 21, 36, 8, 51, 16, 71, 20, 80, 20);
var curPlayer = 0;

function nextTurn() {
	document.getElementById(players[curPlayer].name + "_row").style.backgroundColor = "transparent";
	for(var c = 0; c < players.length; c++) {
		curPlayer = (curPlayer + 1) % players.length;
		if(players[curPlayer].position < 100) {
			document.getElementById(players[curPlayer].name + "_row").style.backgroundColor = "#DDDDDD";
			rollr.style.backgroundColor = players[curPlayer].color;
			if(players[curPlayer].auto) {
					var i = curPlayer;
					curPlayer = -1;
					setTimeout("curPlayer="+i+";roll();", 1000); }
			return; } }
		
		if(players.length > 1) alert("Hooray! Everyone is a winner!");
		curPlayer = -1;
		return; }

function getCompetitiveRoll(position) {
			var attempts = 0;
			var roll;
			var competitiveMode = document.getElementById("competitiveAI").checked;
		
			if (competitiveMode) {
				do {
					roll = Math.floor(Math.random() * 6) + 1;
					var tempPos = position + roll;
					var willFall = false;
		
					//Check if the new position will land on a chute
					for (var i = 0; i < chutes.length; i += 2) {
						if (chutes[i] == tempPos) {
							willFall = true;
							break;
						}
					}
					attempts++;
				//Max 3 attempts to avoid chutes
				} while (willFall && attempts < 3); 
		
				return roll;
			} else {
				//If not in competitive mode, return a regular dice roll
				return Math.floor(Math.random() * 6) + 1;
			}
		}
		
function toggleFastPlay() {
    	fastPlay = document.getElementById("fast-play-toggle").checked;
}
function roll() {
    if (curPlayer == -1) return;
    var curTurn = players[curPlayer];
    var curPiece = document.getElementById(curTurn.name + "_piece");
    var curData = document.getElementById(curTurn.name + "_row");
    var dce = document.getElementById("dice");
    var fastPlay = document.getElementById("fast-play-toggle")?.checked;
    var normalSpeed = 300;
    var fastSpeed = 100;
    var slideSpeed = 2000;
    var animationSpeed = fastPlay ? fastSpeed : normalSpeed;

    curPiece.style.zIndex = players.length;

    //Adjust zIndex for overlapping pieces
    for (var i = 1; i < players.length; i++) {
        var thatPlayer = players[(curPlayer + i) % players.length];
        if (thatPlayer.position < 100)
            document.getElementById(thatPlayer.name + "_piece").style.zIndex--;
    }

    //Play the dice sound effect
    var diceSound = document.getElementById("dice-sound");
    diceSound.currentTime = 0;
    diceSound.play();

    //Dice roll animation
    var rollCount = 0;
    var animateDice = function() {
        rollCount++;
        var tempNum = Math.floor(Math.random() * 6) + 1;
        dce.src = tempNum + "_dice.gif";
        dce.alt = "rolling...";
        if (rollCount < 10) {
            setTimeout(animateDice, 100);
        } else {
            dce.src = rNum + "_dice.gif";
            dce.alt = "roll of " + rNum;
            movePiece();
        }
    };

    var rNum = curTurn.auto ? getCompetitiveRoll(curTurn.position) : Math.floor(Math.random() * 6) + 1;

    //Animate piece movement
    var steps = 0;
    var movePiece = function() {
        if (steps < rNum) {
            curTurn.position++;
            curPiece.style.transition = ""; //Reset transition for normal movement
            curPiece.style.top = getTop(curTurn.position);
            curPiece.style.left = getLeft(curTurn.position);
            curData.childNodes[1].firstChild.nodeValue = curTurn.position;
            steps++;
            setTimeout(movePiece, animationSpeed);
        } else {
            //Check for chutes or ladders with sound effects
            var slideDetected = false;
            for (var i = 0; i < chutes.length; i += 2) {
                if (chutes[i] == curTurn.position || ladders[i] == curTurn.position) {
                    var thisPlayer = curPlayer;
                    curPlayer = -1;
                    if (chutes[i] == curTurn.position) {
                        if (!curTurn.auto) 
                            alert("You landed on a chute and fell " + chutes[i + 1] + " spaces");
                        curTurn.position -= chutes[i + 1];
                        slideDetected = true;
                        document.getElementById("chute-sound").play();
                    } else if (curTurn.auto || confirm("You landed on a " + ladders[i + 1] + " space ladder. Do you want to climb it?")) {
                        curTurn.position += ladders[i + 1];
                        slideDetected = true;
                        document.getElementById("ladder-sound").play();
                    }
                    curPlayer = thisPlayer;

                    //Slide animation for chutes and ladders
                    curPiece.style.transition = `top ${slideSpeed}ms, left ${slideSpeed}ms`;
                    curPiece.style.top = getTop(curTurn.position);
                    curPiece.style.left = getLeft(curTurn.position);
                    curData.childNodes[1].firstChild.nodeValue = curTurn.position;

                    //Reset transition after sliding
                    setTimeout(() => {
                        curPiece.style.transition = ""; 
                    }, slideSpeed);
                }
            }

            //Handle winning condition
            if (curTurn.position >= 100) {
                curTurn.position = 100;
                curPiece.style.top = getTop(curTurn.position);
                curPiece.style.left = getLeft(curTurn.position);
                curData.childNodes[1].firstChild.nodeValue = curTurn.position;
                
				//Play Victory Cheer sound effect
				var cheerSound = document.getElementById("victory-sound");
				cheerSound.currentTime = 0;
				cheerSound.play();
			
				//Trigger Confetti and Message after sound starts
				setTimeout(function() {
					celebrateWinner(curTurn.name);
				}, 500); //Adjust this timing if needed
			} else {
				nextTurn();
			}
        }
    };

    animateDice();
}	

function newPlayer(auto) {
	this.auto = (auto == null)? auto=false : auto;
	if(!auto) {
		if(document.getElementById("name").value == "") return;
		if(document.getElementById(document.getElementById("name").value + "_piece")) {
			alert("Please enter a new unique name");
			return;	}
		this.name = document.getElementById("name").value; }
	else this.name = "Computer" + players.length;
	document.getElementById("name").value = "";
	this.position = 0;
	this.color = colors[players.length % colors.length];
	var playingPiece = document.createElement("img");
	playingPiece.id = this.name + "_piece";
	playingPiece.src = this.color + ".png";
	playingPiece.alt = this.name + "\'s playing piece";
	playingPiece.allowtransparency = "true";
	playingPiece.style.position = "absolute";
	playingPiece.style.top = getTop(0);
	playingPiece.style.left = getLeft(0);
	playingPiece.width = playingPiece.height = getPieceSize();
	var board = document.getElementById("board");
	board.appendChild(playingPiece);
	players[players.length] = this;
	var table = document.getElementById("players");
	var newRow = table.insertRow(table.rows.length);
	newRow.id = this.name + "_row";
	newRow.style.color = this.color;
	var nameCol = document.createElement("td");
	nameCol.appendChild(document.createTextNode(this.name));
	var scoreCol = document.createElement("td");
	scoreCol.appendChild(document.createTextNode("0"));
	newRow.appendChild(nameCol);
	newRow.appendChild(scoreCol);
	if(players.length == 1) {
		rollr.style.visibility = "visible";
		rollr.style.backgroundColor = this.color;
		newRow.style.backgroundColor = "#DDDDDD"; }
	else if(curPlayer == -1) {
		curPlayer = players.length - 1;
		nextTurn(); } }

function getTop(pos) {
	if(pos == 0) return "0px";
	var picHeight = document.getElementById("picture").height;
	return .98 * (1.02* picHeight - Math.floor((pos-1)/10 + 1) * picHeight/10 + Math.random()*0.05*picHeight) + "px"; }
function getLeft(pos) {
	if(pos == 0) return (0-getPieceSize()) + "px";
	var picWidth = document.getElementById("picture").width;
	var numSecs = pos % 20;
	if(numSecs == 0) numSecs = 1;
	if(numSecs > 10) numSecs = 21 - numSecs;
	return .98 * (0.02*picWidth + (numSecs-1)*picWidth/10 + Math.random()*0.05*picWidth) + "px"; }
function getPieceSize() {
	return (document.getElementById("picture").height)/20; }
function resize() {
	pic = document.getElementById("picture");
	var lft = document.getElementById("left");
	var hgt = document.documentElement.clientHeight - document.getElementById("board").offsetTop;
	var wdt = document.documentElement.clientWidth - document.getElementById("board").offsetLeft;
	pic.height = pic.width = 0.97 * ((hgt > wdt)? wdt : hgt);
	pic.style.border = 0.01*pic.height + "px solid #FF7777";
	for(var i = 0; i < players.length; i++) {
		var playpiece = document.getElementById(players[i].name + "_piece");
		playpiece.height = playpiece.width = getPieceSize();
		playpiece.style.top = getTop(players[i].position);
		playpiece.style.left = getLeft(players[i].position); } }

function celebrateWinner(name) {
			var confettiContainer = document.createElement("div");
			confettiContainer.id = "confettiContainer";
			document.body.appendChild(confettiContainer);
		
			for (var i = 0; i < 100; i++) {
				var confetti = document.createElement("div");
				confetti.classList.add("confetti");
				confetti.style.left = Math.random() * 100 + "vw";
				confetti.style.animationDuration = (Math.random() * 2 + 3) + "s";
				confettiContainer.appendChild(confetti);
			}
		
			//Show congratulatory message
			document.getElementById("winnerName").innerText = name + " is the winner!";
			document.getElementById("congratsMessage").style.display = "block";
            }
		
function closeCongrats() {
			document.getElementById("congratsMessage").style.display = "none";
			var confettiContainer = document.getElementById("confettiContainer");
			if (confettiContainer) {
				confettiContainer.remove();
			}
		}
		
function toggleMusic() {
			var bgMusic = document.getElementById("bg-music");
			var toggleButton = document.getElementById("music-toggle");
		
			if (bgMusic.paused || bgMusic.currentTime === 0) {
				bgMusic.play().catch(error => {
					console.error("Music playback error:", error);
				});
				toggleButton.textContent = "Music: On";
				toggleButton.classList.add("active");
			} else {
				bgMusic.pause();
				toggleButton.textContent = "Music: Off";
				toggleButton.classList.remove("active");
			}
		}
		
		
function grabber(event) {
	diffX = event.clientX - parseInt(rollr.style.left);
	diffY = event.clientY - parseInt(rollr.style.top);
	addEventListener("mousemove", mover, true);
	addEventListener("mouseup", dropper, true);
	event.stopPropagation(); }
function mover(event) {
	rollr.style.left = (event.clientX - diffX) + "px";
	rollr.style.top = (event.clientY - diffY) + "px"; }
function dropper(event) {
	removeEventListener("mouseup", dropper, true);
	removeEventListener("mousemove", mover, true); }

function addEventListener(a,b,c) {
	if(document.addEventListener) document.addEventListener(a,b,c);
	else document.attachEvent("on"+a,b); }
function removeEventListener(a,b,c) {
	if(document.removeEventListener) document.removeEventListener(a,b,c);
	else document.detachEvent("on"+a,b); }
function initialize() {
	rollr = document.getElementById("roller");
	rollr.style.zIndex = 1000;
	document.getElementById("picture").style.zIndex = 0;
	window.onresize=resize;
	resize(); }
window.onload=initialize;
